@@ -21,14 +21,16 @@
    Author: Stef Walter <stef@memberwebs.com>
 */
 
+#include "config.h"
+
+#include "gnome-keyring.h"
+#include <glib.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
-
-#include "run-auto-test.h"
-
-#include "library/gnome-keyring.h"
+#include <unistd.h>
 
 static GList* keyrings = NULL;
 
@@ -38,7 +40,29 @@ static GList* keyrings = NULL;
 #define DISPLAY_NAME "Item Display Name"
 #define SECRET "item-secret"
 
-DEFINE_TEST(remove_incomplete)
+static GMainLoop *mainloop = NULL;
+
+static gboolean
+quit_loop (gpointer data)
+{
+	g_main_loop_quit (data);
+	return TRUE;
+}
+
+static void
+mainloop_run (int timeout)
+{
+	guint id = 0;
+
+	if (timeout)
+		id = g_timeout_add (timeout, quit_loop, mainloop);
+	g_main_loop_run (mainloop);
+	if (timeout)
+		g_source_remove (id);
+}
+
+static void
+test_remove_incomplete (void)
 {
 	GnomeKeyringResult res;
 
@@ -47,7 +71,8 @@ DEFINE_TEST(remove_incomplete)
 		g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, res);
 }
 
-DEFINE_TEST(create_keyring)
+static void
+test_create_keyring (void)
 {
 	GnomeKeyringResult res;
 
@@ -55,7 +80,8 @@ DEFINE_TEST(create_keyring)
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, res);
 }
 
-DEFINE_TEST(create_keyring_already_exists)
+static void
+test_create_keyring_already_exists (void)
 {
 	GnomeKeyringResult res;
 
@@ -64,7 +90,8 @@ DEFINE_TEST(create_keyring_already_exists)
 	g_assert_cmpint (GNOME_KEYRING_RESULT_ALREADY_EXISTS, ==, res);
 }
 
-DEFINE_TEST(set_default_keyring)
+static void
+test_set_default_keyring (void)
 {
 	GnomeKeyringResult res;
 	gchar* name;
@@ -81,7 +108,8 @@ DEFINE_TEST(set_default_keyring)
 	g_assert_cmpstr (name, ==, KEYRING_NAME);
 }
 
-DEFINE_TEST(delete_keyring)
+static void
+test_delete_keyring (void)
 {
 	GnomeKeyringResult res;
 	gchar* name;
@@ -98,7 +126,8 @@ DEFINE_TEST(delete_keyring)
 	g_assert(name == NULL || strcmp (name, KEYRING_NAME) != 0);
 }
 
-DEFINE_TEST(recreate_keyring)
+static void
+test_recreate_keyring (void)
 {
 	GnomeKeyringResult res;
 
@@ -117,7 +146,8 @@ compare_ids (gconstpointer a, gconstpointer b)
 	return a == b ? 0 : 1;
 }
 
-DEFINE_TEST(create_list_items)
+static void
+test_create_list_items (void)
 {
 	GnomeKeyringResult res;
 	guint id, id2, id3;
@@ -203,7 +233,8 @@ DEFINE_TEST(create_list_items)
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, res);
 }
 
-DEFINE_TEST(find_keyrings)
+static void
+test_find_keyrings (void)
 {
 	GnomeKeyringResult res;
 	GnomeKeyringAttributeList* attrs;
@@ -268,7 +299,8 @@ DEFINE_TEST(find_keyrings)
  * A find that does not match should return 'Not Found':
  * http://bugzilla.gnome.org/show_bug.cgi?id=476682
  */
-DEFINE_TEST(find_invalid)
+static void
+test_find_invalid (void)
 {
 	GnomeKeyringResult res;
 	GnomeKeyringAttributeList* attrs;
@@ -282,7 +314,8 @@ DEFINE_TEST(find_invalid)
 	g_assert_cmpint (GNOME_KEYRING_RESULT_NO_MATCH, ==, res);
 }
 
-DEFINE_TEST(lock_keyrings)
+static void
+test_lock_keyrings (void)
 {
 	GnomeKeyringResult res;
 
@@ -300,7 +333,8 @@ DEFINE_TEST(lock_keyrings)
 	g_assert_cmpint (GNOME_KEYRING_RESULT_NO_SUCH_KEYRING, ==, res);
 }
 
-DEFINE_TEST(change_password)
+static void
+test_change_password (void)
 {
 	GnomeKeyringResult res;
 
@@ -308,7 +342,8 @@ DEFINE_TEST(change_password)
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, res);
 }
 
-DEFINE_TEST(keyring_info)
+static void
+test_keyring_info (void)
 {
 	GnomeKeyringResult res;
 	GnomeKeyringInfo *info;
@@ -320,7 +355,8 @@ DEFINE_TEST(keyring_info)
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, res);
 }
 
-DEFINE_TEST(list_keyrings)
+static void
+test_list_keyrings (void)
 {
 	GnomeKeyringResult res;
 	GList *l;
@@ -339,10 +375,11 @@ static void
 done_grant_access (GnomeKeyringResult res, gpointer data)
 {
 	grant_access_result = res;
-	test_mainloop_quit ();
+	g_main_loop_quit (mainloop);
 }
 
-DEFINE_TEST(keyring_grant_access)
+static void
+test_keyring_grant_access (void)
 {
 	GList *acl;
 	GnomeKeyringResult res;
@@ -365,7 +402,7 @@ DEFINE_TEST(keyring_grant_access)
 	/* "callback already called" */
 	g_assert_cmpint (grant_access_result, ==, GNOME_KEYRING_RESULT_CANCELLED);
 
-	test_mainloop_run (2000);
+	mainloop_run (2000);
 
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, grant_access_result);
 
@@ -393,10 +430,11 @@ static void
 done_store_password (GnomeKeyringResult res, gpointer data)
 {
 	*((GnomeKeyringResult*)data) = res;
-	test_mainloop_quit ();
+	g_main_loop_quit (mainloop);
 }
 
-DEFINE_TEST(store_password)
+static void
+test_store_password (void)
 {
 	GnomeKeyringResult res;
 	gpointer op;
@@ -428,7 +466,7 @@ DEFINE_TEST(store_password)
 	/* "callback already called" */
 	g_assert_cmpint (res, ==, GNOME_KEYRING_RESULT_CANCELLED);
 
-	test_mainloop_run (2000);
+	mainloop_run (2000);
 
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, res);
 }
@@ -447,10 +485,11 @@ done_find_password (GnomeKeyringResult res, const gchar* password, gpointer unus
 		g_assert_cmpstr (password, ==, "password");
 	}
 
-	test_mainloop_quit ();
+	g_main_loop_quit (mainloop);
 }
 
-DEFINE_TEST(find_password)
+static void
+test_find_password (void)
 {
 	GnomeKeyringResult res;
 	gchar *password;
@@ -484,7 +523,7 @@ DEFINE_TEST(find_password)
 	/* "callback already called" */
 	g_assert (find_password_result == GNOME_KEYRING_RESULT_CANCELLED);
 
-	test_mainloop_run (2000);
+	mainloop_run (2000);
 
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, find_password_result);
 }
@@ -496,10 +535,11 @@ done_find_no_password (GnomeKeyringResult res, const gchar* password, gpointer u
 {
 	find_no_password_result = res;
 	g_assert (password == NULL);
-	test_mainloop_quit ();
+	g_main_loop_quit (mainloop);
 }
 
-DEFINE_TEST(find_no_password)
+static void
+test_find_no_password (void)
 {
 	GnomeKeyringResult res;
 	gchar *password = NULL;
@@ -523,7 +563,7 @@ DEFINE_TEST(find_no_password)
 	g_assert (op != NULL);
 	g_assert (find_no_password_result == GNOME_KEYRING_RESULT_CANCELLED);
 
-	test_mainloop_run (2000);
+	mainloop_run (2000);
 
 	g_assert_cmpint (GNOME_KEYRING_RESULT_NO_MATCH, ==, find_no_password_result);
 }
@@ -532,10 +572,11 @@ static void
 done_delete_password (GnomeKeyringResult res, gpointer data)
 {
 	*((GnomeKeyringResult*)data) = res;
-	test_mainloop_quit ();
+	g_main_loop_quit (mainloop);
 }
 
-DEFINE_TEST(delete_password)
+static void
+test_delete_password (void)
 {
 	GnomeKeyringResult res;
 	gpointer op;
@@ -562,16 +603,155 @@ DEFINE_TEST(delete_password)
 	/* "callback already called" */
 	g_assert (res == GNOME_KEYRING_RESULT_CANCELLED);
 
-	test_mainloop_run (2000);
+	mainloop_run (2000);
 
 	/* Should have already been deleted by the second call above */
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, res);
 }
 
-DEFINE_TEST(cleanup)
+static void
+test_cleanup (void)
 {
 	GnomeKeyringResult res;
 
 	res = gnome_keyring_delete_sync (KEYRING_NAME);
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, res);
 }
+
+static void
+test_set_display (void)
+{
+	GnomeKeyringResult res;
+
+	/* Deprecated method */
+	res = gnome_keyring_daemon_set_display_sync (":0.0");
+	g_assert_cmpint (GNOME_KEYRING_RESULT_DENIED, ==, res);
+}
+
+static void
+test_setup_environment (void)
+{
+	GnomeKeyringResult res;
+
+	res = gnome_keyring_daemon_prepare_environment_sync ();
+	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, res);
+}
+
+static pid_t daemon_pid = 0;
+
+static gboolean
+daemon_start (void)
+{
+	GError *err = NULL;
+	gchar *args[5];
+	const gchar *path, *service, *address;
+	gchar *output = NULL;
+	gint exit_status = 1;
+	GError *error = NULL;
+
+	/* Need to have DBUS running */
+	address = g_getenv ("DBUS_SESSION_BUS_ADDRESS");
+	if (!address || !address[0]) {
+		g_printerr ("\nNo DBUS session available, skipping tests!\n\n");
+		return FALSE;
+	}
+
+	/* Check if gnome-keyring-daemon has testing enabled */
+	if (!g_spawn_command_line_sync (GNOME_KEYRING_DAEMON_PATH " --version",
+	                                &output, NULL, &exit_status, &error)) {
+		g_printerr ("Couldn't launch '%s': %s", GNOME_KEYRING_DAEMON_PATH,
+		            error->message);
+		return FALSE;
+	}
+
+	/* For some reason --version failed */
+	if (exit_status != 0 || strstr (output, "testing: enabled") == NULL) {
+		g_printerr ("Skipping tests since no testing enabled gnome-keyring-daemin is available\n");
+		g_free (output);
+		return FALSE;
+	}
+
+	g_free (output);
+
+	path = g_getenv ("GNOME_KEYRING_TEST_PATH");
+	if (path && !path[0])
+		path = NULL;
+
+	service = g_getenv ("GNOME_KEYRING_TEST_SERVICE");
+	if (service && !service[0])
+		service = NULL;
+
+	if (!path && !service) {
+		g_mkdir_with_parents ("/tmp/keyring-test/data", 0700);
+		g_setenv ("GNOME_KEYRING_TEST_PATH", "/tmp/keyring-test/data", TRUE);
+		g_setenv ("GNOME_KEYRING_TEST_SERVICE", "org.gnome.keyring.Test", TRUE);
+
+		g_printerr ("Starting gnome-keyring-daemon...\n");
+
+		args[0] = GNOME_KEYRING_DAEMON_PATH;
+		args[1] = "--foreground";
+		args[2] = "--control-directory";
+		args[3] = "/tmp/keyring-test";
+		args[4] = NULL;
+
+		if (!g_spawn_async (NULL, args, NULL, G_SPAWN_LEAVE_DESCRIPTORS_OPEN | G_SPAWN_DO_NOT_REAP_CHILD,
+							NULL, NULL, &daemon_pid, &err)) {
+			g_error ("couldn't start gnome-keyring-daemon for testing: %s",
+					 err && err->message ? err->message : "");
+			g_assert_not_reached ();
+		}
+
+		/* Let it startup properly */
+		sleep (2);
+	}
+
+	return TRUE;
+}
+
+static void
+daemon_stop (void)
+{
+	if (daemon_pid)
+		kill (daemon_pid, SIGTERM);
+	daemon_pid = 0;
+}
+
+int
+main (int argc, char **argv)
+{
+	int ret = 0;
+
+	g_test_init (&argc, &argv, NULL);
+	g_set_prgname ("test-keyrings");
+
+	mainloop = g_main_loop_new (NULL, FALSE);
+
+	g_test_add_func ("/keyrings/remove-incomplete", test_remove_incomplete);
+	g_test_add_func ("/keyrings/create-keyring", test_create_keyring);
+	g_test_add_func ("/keyrings/create-keyring-already-exists", test_create_keyring_already_exists);
+	g_test_add_func ("/keyrings/set-default-keyring", test_set_default_keyring);
+	g_test_add_func ("/keyrings/delete-keyring", test_delete_keyring);
+	g_test_add_func ("/keyrings/recreate-keyring", test_recreate_keyring);
+	g_test_add_func ("/keyrings/create-list-items", test_create_list_items);
+	g_test_add_func ("/keyrings/find-keyrings", test_find_keyrings);
+	g_test_add_func ("/keyrings/find-invalid", test_find_invalid);
+	g_test_add_func ("/keyrings/lock-keyrings", test_lock_keyrings);
+	g_test_add_func ("/keyrings/change-password", test_change_password);
+	g_test_add_func ("/keyrings/keyring-info", test_keyring_info);
+	g_test_add_func ("/keyrings/list-keyrings", test_list_keyrings);
+	g_test_add_func ("/keyrings/keyring-grant-access", test_keyring_grant_access);
+	g_test_add_func ("/keyrings/store-password", test_store_password);
+	g_test_add_func ("/keyrings/find-password", test_find_password);
+	g_test_add_func ("/keyrings/find-no-password", test_find_no_password);
+	g_test_add_func ("/keyrings/delete-password", test_delete_password);
+	g_test_add_func ("/keyrings/cleanup", test_cleanup);
+	g_test_add_func ("/keyrings/set-display", test_set_display);
+	g_test_add_func ("/keyrings/setup-environment", test_setup_environment);
+
+	if (daemon_start ()) {
+		ret = g_test_run ();
+		daemon_stop ();
+	}
+
+	return ret;
+}
